@@ -38,7 +38,7 @@
  * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $Id: idle_idled.c,v 1.1 2000/12/14 19:26:48 ken3 Exp $ */
+/* $Id: idle_idled.c,v 1.4 2001/01/19 16:48:58 ken3 Exp $ */
 
 #include <sys/types.h>
 #include <sys/socket.h>
@@ -46,8 +46,11 @@
 #include <syslog.h>
 #include <sys/stat.h>
 #include <fcntl.h>
+#include <syslog.h>
 #include <time.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 #include <signal.h>
 
 #include "idle.h"
@@ -134,16 +137,32 @@ void idle_poll(int sig)
 
 int idle_init(struct mailbox *mailbox, idle_updateproc_t *proc)
 {
+    struct sigaction action;
+
     idle_update = proc;
 
-    /* Tell idling that we're idling */
+    sigemptyset(&action.sa_mask);
+    action.sa_flags = 0;
+#ifdef SA_RESTART
+    action.sa_flags |= SA_RESTART;
+#endif
+    action.sa_handler = idle_poll;
+
+    /* Tell idled that we're idling */
     if (idle_send_msg(IDLE_INIT, mailbox)) {
 	/* if we can talk to idled, setup the signal handlers */
-	signal(SIGUSR1, idle_poll);
-	signal(SIGUSR2, idle_poll);
+	if ((sigaction(SIGUSR1, &action, NULL) < 0) ||
+	    (sigaction(SIGUSR2, &action, NULL) < 0)) {
+	    syslog(LOG_ERR, "sigaction: %m");
+	    return 0;
+	}
     }
     else { /* otherwise, we'll poll with SIGALRM */
-	signal(SIGALRM, idle_poll);
+	if (sigaction(SIGALRM, &action, NULL) < 0) {
+	    syslog(LOG_ERR, "sigaction: %m");
+	    return 0;
+	}
+
 	alarm(idle_period);
     }
 
@@ -156,9 +175,9 @@ void idle_done(struct mailbox *mailbox)
     idle_send_msg(IDLE_DONE, mailbox);
 
     /* Remove the signal handlers */
-    signal(SIGUSR1, SIG_DFL);
-    signal(SIGUSR2, SIG_DFL);
-    signal(SIGALRM, SIG_DFL);
+    signal(SIGUSR1, SIG_IGN);
+    signal(SIGUSR2, SIG_IGN);
+    signal(SIGALRM, SIG_IGN);
 }
 
 /*
@@ -174,10 +193,9 @@ int idle_send_msg(int msg, struct mailbox *mailbox)
     strcpy(idledata.mboxname, mailbox ? mailbox->name : ".");
 
     /* send */
-    if (sendto(notify_sock, &idledata,
-	       IDLEDATA_BASE_SIZE+strlen(idledata.mboxname), 0,
-	       (struct sockaddr *) &idle_remote, 
-	       idle_remote_len) == -1) {
+    if (sendto(notify_sock, (void *) &idledata,
+	       IDLEDATA_BASE_SIZE+strlen(idledata.mboxname)+1, /* 1 for NULL */
+	       0, (struct sockaddr *) &idle_remote, idle_remote_len) == -1) {
       syslog(LOG_ERR, "error sending to idled: %x", msg);
       return 0;
     }
