@@ -1,5 +1,5 @@
 /* lmtpengine.c: LMTP protocol engine
- * $Id: lmtpengine.c,v 1.18 2001/01/31 00:59:57 ken3 Exp $
+ * $Id: lmtpengine.c,v 1.21 2001/02/19 18:05:37 leg Exp $
  *
  * Copyright (c) 2000 Carnegie Mellon University.  All rights reserved.
  *
@@ -83,7 +83,7 @@
 #include "lmtpengine.h"
 #include "lmtpstats.h"
 
-#define RCPT_GROW 5 /* XXX 30 */
+#define RCPT_GROW 30
 
 /* data per message */
 struct Header {
@@ -968,6 +968,7 @@ void lmtpmode(struct lmtp_func *func,
     socklen_t salen;
 
     sasl_conn_t *conn = NULL;
+    int secflags = 0;
     sasl_security_properties_t *secprops = NULL;
     sasl_external_properties_t *extprops = NULL;
     int authenticated = 0;	/* -1: external auth'd, but no AUTH issued
@@ -982,7 +983,11 @@ void lmtpmode(struct lmtp_func *func,
     }
 
     /* set my allowable security properties */
-    secprops = mysasl_secprops(SASL_SEC_NOANONYMOUS | SASL_SEC_NOPLAINTEXT);
+    secflags = SASL_SEC_NOANONYMOUS;
+    if (!config_getswitch("allowplaintext", 1)) {
+	secflags |= SASL_SEC_NOPLAINTEXT;
+    }
+    secprops = mysasl_secprops(secflags);
     sasl_setprop(conn, SASL_SEC_PROPS, secprops);
 
     /* determine who we're talking to */
@@ -1222,8 +1227,8 @@ void lmtpmode(struct lmtp_func *func,
 	      char *mechs;
 	      
 	      prot_printf(pout, "250-%s\r\n"
-			  "250-8BITMIME\r\n"
 			  "250-IGNOREQUOTA\r\n"
+			  "250-8BITMIME\r\n"
 			  "250-ENHANCEDSTATUSCODES\r\n",
 			  config_servername);
 	      if (sasl_listmech(conn, NULL, "AUTH ", " ", "", &mechs, 
@@ -1538,9 +1543,14 @@ static void chop(char *s)
     }
 }
 
+/* xxx fill in this function 
+
+   perform authentication against connection 'conn'
+   returns the SMTP error code from the AUTH attempt */
 static int do_auth(struct lmtp_conn *conn)
 {
-    return 0;
+    /* pretend success for now */
+    return 250;
 }
 
 /* establish connection, LHLO, and AUTH if possible */
@@ -1673,7 +1683,7 @@ int lmtp_connect(const char *phost,
 		if (!strcasecmp(buf + 4, "PIPELINING")) {
 		    conn->capability |= CAPA_PIPELINING;
 		}
-		if (!strcasecmp(buf + 4, "AUTH ")) {
+		if (!strncasecmp(buf + 4, "AUTH ", 5)) {
 		    conn->capability |= CAPA_AUTH;
 		    /* save mechanisms for later */
 		    conn->mechs = xstrdup(buf + 9);
