@@ -1,6 +1,6 @@
 /* lmtpproxyd.c -- Program to sieve and proxy mail delivery
  *
- * $Id: lmtpproxyd.c,v 1.8 2000/08/22 14:12:10 ken3 Exp $
+ * $Id: lmtpproxyd.c,v 1.14 2000/12/19 19:31:41 ken3 Exp $
  * Copyright (c) 1999-2000 Carnegie Mellon University.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -42,7 +42,7 @@
  *
  */
 
-/*static char _rcsid[] = "$Id: lmtpproxyd.c,v 1.8 2000/08/22 14:12:10 ken3 Exp $";*/
+/*static char _rcsid[] = "$Id: lmtpproxyd.c,v 1.14 2000/12/19 19:31:41 ken3 Exp $";*/
 
 #include <config.h>
 
@@ -161,7 +161,7 @@ static char *generate_notify(message_data_t *m);
 
 void shut_down(int code);
 
-struct lmtp_func mylmtp = { &deliver, &verify_user, 0 };
+struct lmtp_func mylmtp = { &deliver, &verify_user, 0, 0 };
 
 static int quotaoverride = 0;		/* should i override quota? */
 int dupelim = 0;
@@ -453,7 +453,10 @@ static char *sendmail_errstr(int sm_stat)
 	snprintf(errstr, sizeof errstr,
 		"Sendmail process terminated abnormally, signal = %d %s\n",
 		WTERMSIG(sm_stat), 
-		WCOREDUMP(sm_stat) ? " -- core file generated" : "");
+#ifdef WCOREDUMP
+		WCOREDUMP(sm_stat) ? " -- core file generated" :
+#endif
+		"");
     } else if (WIFSTOPPED(sm_stat)) {
 	snprintf(errstr, sizeof errstr,
 		 "Sendmail process stopped, signal = %d\n",
@@ -554,7 +557,7 @@ int send_rejection(const char *origid,
 	fwrite(buf, i, 1, sm);
     }
     fprintf(sm, "\r\n\r\n");
-    fprintf(sm, "--%d/%s\r\n", (int) p, config_servername);
+    fprintf(sm, "--%d/%s--\r\n", (int) p, config_servername);
 
     fclose(sm);
     while (waitpid(sm_pid, &sm_stat, 0) < 0);
@@ -604,10 +607,26 @@ static
 int sieve_redirect(void *ac, void *ic, void *sc, void *mc, const char **errmsg)
 {
     sieve_redirect_context_t *rc = (sieve_redirect_context_t *) ac;
+    script_data_t *sd = (script_data_t *) sc;
     message_data_t *m = ((mydata_t *) mc)->m;
+    char buf[8192], *sievedb = NULL;
     int res;
 
+    /* if we have a msgid, we can track our redirects */
+    if (m->id) {
+	snprintf(buf, sizeof(buf), "%s-%s", m->id, rc->addr);
+	sievedb = make_sieve_db(sd->username);
+
+	/* ok, let's see if we've redirected this message before */
+	if (duplicate_check(buf, strlen(buf), sievedb, strlen(sievedb)))
+	    return SIEVE_OK;
+    }
+
     if ((res = send_forward(rc->addr, m->return_path, m->data)) == 0) {
+	/* mark this message as redirected */
+	if (sievedb) duplicate_mark(buf, strlen(buf), 
+				    sievedb, strlen(sievedb), time(NULL));
+
 	snmp_increment(SIEVE_REDIRECT, 1);
 	return SIEVE_OK;
     } else {
@@ -780,7 +799,7 @@ int send_response(void *ac, void *ic, void *sc, void *mc, const char **errmsg)
     time_t t;
     pid_t sm_pid, p;
     sieve_send_response_context_t *src = (sieve_send_response_context_t *) ac;
-    message_data_t *m = (message_data_t *) mc;
+    message_data_t *md = ((mydata_t *) mc)->m;
     script_data_t *sdata = (script_data_t *) sc;
 
     smbuf[0] = "sendmail";
@@ -826,7 +845,7 @@ int send_response(void *ac, void *ic, void *sc, void *mc, const char **errmsg)
 	    break;
 	}
     fprintf(sm, "Subject: %s\r\n", src->subj);
-    fprintf(sm, "In-Reply-To: %s\r\n", m->id);
+    if (md->id) fprintf(sm, "In-Reply-To: %s\r\n", md->id);
     fprintf(sm, "Auto-Submitted: auto-replied (vacation)\r\n");
     if (src->mime) {
 	fprintf(sm, "MIME-Version: 1.0\r\n");
@@ -841,7 +860,7 @@ int send_response(void *ac, void *ic, void *sc, void *mc, const char **errmsg)
     fprintf(sm, "%s\r\n", src->msg);
 
     if (src->mime) {
-	fprintf(sm, "\r\n--%d/%s\r\n", (int) p, config_servername);
+	fprintf(sm, "\r\n--%d/%s--\r\n", (int) p, config_servername);
     }
     fclose(sm);
     while (waitpid(sm_pid, &sm_stat, 0) < 0);
@@ -869,7 +888,7 @@ sieve_vacation_t vacation = {
     &send_response,		/* send_response() */
 };
 
-int sieve_parse_error_handler(int lineno, char *msg, void *ic, void *sc)
+int sieve_parse_error_handler(int lineno, const char *msg, void *ic, void *sc)
 {
     script_data_t *sd = (script_data_t *) sc;
     
@@ -879,7 +898,7 @@ int sieve_parse_error_handler(int lineno, char *msg, void *ic, void *sc)
     return SIEVE_OK;
 }
 
-int sieve_execute_error_handler(char *msg, void *ic, void *sc, void *mc)
+int sieve_execute_error_handler(const char *msg, void *ic, void *sc, void *mc)
 {
     script_data_t *sd = (script_data_t *) sc;
     message_data_t *md = ((mydata_t *) mc)->m;
