@@ -1,32 +1,50 @@
 /* seen_local.c -- Storage for /Recent and /Seen state on local filesystem
- $Id: seen_local.c,v 1.27 1999/10/02 00:43:07 leg Exp $
- 
- # Copyright 1998 Carnegie Mellon University
- # 
- # No warranties, either expressed or implied, are made regarding the
- # operation, use, or results of the software.
- #
- # Permission to use, copy, modify and distribute this software and its
- # documentation is hereby granted for non-commercial purposes only
- # provided that this copyright notice appears in all copies and in
- # supporting documentation.
- #
- # Permission is also granted to Internet Service Providers and others
- # entities to use the software for internal purposes.
- #
- # The distribution, modification or sale of a product which uses or is
- # based on the software, in whole or in part, for commercial purposes or
- # benefits requires specific, additional permission from:
- #
- #  Office of Technology Transfer
- #  Carnegie Mellon University
- #  5000 Forbes Avenue
- #  Pittsburgh, PA  15213-3890
- #  (412) 268-4387, fax: (412) 268-7395
- #  tech-transfer@andrew.cmu.edu
+ $Id: seen_local.c,v 1.32 2000/05/23 20:52:31 robeson Exp $
+ * Copyright (c) 1998-2000 Carnegie Mellon University.  All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer. 
+ *
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
+ *
+ * 3. The name "Carnegie Mellon University" must not be used to
+ *    endorse or promote products derived from this software without
+ *    prior written permission. For permission or any other legal
+ *    details, please contact  
+ *      Office of Technology Transfer
+ *      Carnegie Mellon University
+ *      5000 Forbes Avenue
+ *      Pittsburgh, PA  15213-3890
+ *      (412) 268-4387, fax: (412) 268-7395
+ *      tech-transfer@andrew.cmu.edu
+ *
+ * 4. Redistributions of any form whatsoever must retain the following
+ *    acknowledgment:
+ *    "This product includes software developed by Computing Services
+ *     at Carnegie Mellon University (http://www.cmu.edu/computing/)."
+ *
+ * CARNEGIE MELLON UNIVERSITY DISCLAIMS ALL WARRANTIES WITH REGARD TO
+ * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
+ * AND FITNESS, IN NO EVENT SHALL CARNEGIE MELLON UNIVERSITY BE LIABLE
+ * FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
+ * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN
+ * AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING
+ * OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
  *
  */
 
+#include <config.h>
+
+#ifdef HAVE_UNISTD_H
+#include <unistd.h>
+#endif
 #include <stdio.h>
 #include <stdlib.h>
 #include <errno.h>
@@ -48,6 +66,8 @@
 #include "imap_err.h"
 #include "xmalloc.h"
 
+#include "seen.h"
+
 extern int errno;
 
 #define FNAME_SEEN "/cyrus.seen"
@@ -68,10 +88,7 @@ struct seen {
  * Returns pointer to abstract database type in buffer pointed to
  * by 'seendbptr'.
  */
-int seen_open(mailbox, user, seendbptr)
-struct mailbox *mailbox;
-const char *user;
-struct seen **seendbptr;
+int seen_open(struct mailbox *mailbox, const char *user, struct seen **seendbptr)
 {
     struct seen *seendb;
     char fnamebuf[MAX_MAILBOX_PATH];
@@ -118,12 +135,8 @@ struct seen **seendbptr;
  * 'lastuidptr', and 'seenuidsptr'.  A malloc'ed string is placed in
  * the latter and the caller is responsible for freeing it.
  */
-int seen_lockread(seendb, lastreadptr, lastuidptr, lastchangeptr, seenuidsptr)
-struct seen *seendb;
-time_t *lastreadptr;
-unsigned *lastuidptr;
-time_t *lastchangeptr;
-char **seenuidsptr;
+int seen_lockread(struct seen *seendb, time_t *lastreadptr, unsigned int *lastuidptr, 
+		  time_t *lastchangeptr, char **seenuidsptr)
 {
     int r;
     char fnamebuf[MAX_MAILBOX_PATH];
@@ -174,7 +187,7 @@ char **seenuidsptr;
     left = length - namelen;
 
     /* Parse last-read timestamp */
-    while (left && isdigit(*buf)) {
+    while (left && isdigit((int) *buf)) {
 	*lastreadptr = *lastreadptr * 10 + *buf++ - '0';
 	left--;
     }
@@ -184,7 +197,7 @@ char **seenuidsptr;
     }
 
     /* Parse last-read uid */
-    while (left && isdigit(*buf)) {
+    while (left && isdigit((int) *buf)) {
 	*lastuidptr = *lastuidptr * 10 + *buf++ - '0';
 	left--;
     }
@@ -195,12 +208,12 @@ char **seenuidsptr;
 
     /* Scan for end of uids or last-change timestamp */
     p = buf;
-    while (left && !isspace(*p)) {
+    while (left && !isspace((int) *p)) {
 	p++;
 	left--;
     }
 
-    if (left > 1 && p[0] == ' ' && isdigit(p[1])) {
+    if (left > 1 && p[0] == ' ' && isdigit((int) p[1])) {
 	/* Have a last-change timestamp */
 	while (buf < p) {
 	    *lastchangeptr = *lastchangeptr * 10 + *buf++ - '0';
@@ -210,7 +223,7 @@ char **seenuidsptr;
 	left--;
 
 	/* Scan for end of uids */
-	while (left && !isspace(*p)) {
+	while (left && !isspace((int) *p)) {
 	    p++;
 	    left--;
 	}
@@ -228,16 +241,12 @@ char **seenuidsptr;
  * Write out new data for the user
  */
 #define PADSIZE 30
-int seen_write(seendb, lastread, lastuid, lastchange, seenuids)
-struct seen *seendb;
-time_t lastread;
-unsigned lastuid;
-time_t lastchange;
-char *seenuids;
+int seen_write(struct seen *seendb, time_t lastread, unsigned int lastuid, 
+	       time_t lastchange, char *seenuids)
 {
     char timeuidbuf[80];
     int length;
-    int writefd;
+    int writefd = -1;
     int replace;
     char fnamebuf[MAX_MAILBOX_PATH];
     char newfnamebuf[MAX_MAILBOX_PATH];
@@ -261,7 +270,7 @@ char *seenuids;
 
     assert(seendb->mailbox->seen_lock_count != 0);
 
-    sprintf(timeuidbuf, "\t%u %u %u ", lastread, lastuid, lastchange);
+    sprintf(timeuidbuf, "\t%u %u %u ", (unsigned int) lastread, lastuid, (unsigned int) lastchange);
     
     length = strlen(seendb->user)+strlen(timeuidbuf)+strlen(seenuids)+1;
 
@@ -345,8 +354,7 @@ char *seenuids;
 /*
  * Unlock the database
  */
-int seen_unlock(seendb)
-struct seen *seendb;
+int seen_unlock(struct seen *seendb)
 {
     int r;
 
@@ -366,8 +374,7 @@ struct seen *seendb;
 /*
  * Close the database
  */
-int seen_close(seendb)
-struct seen *seendb;
+int seen_close(struct seen *seendb)
 {
     map_free(&seendb->base, &seendb->size);
     close(seendb->fd);
@@ -379,9 +386,7 @@ struct seen *seendb;
 /*
  * Make the \Seen database for the newly created mailbox 'mailbox'.
  */
-int
-seen_create(mailbox)
-struct mailbox *mailbox;
+int seen_create(struct mailbox *mailbox)
 {
     char fnamebuf[MAX_MAILBOX_PATH];
     int fd;
@@ -401,9 +406,7 @@ struct mailbox *mailbox;
 /*
  * Remove the \Seen database for the mailbox 'mailbox'.
  */
-int
-seen_delete(mailbox)
-struct mailbox *mailbox;
+int seen_delete(struct mailbox *mailbox)
 {
     char fnamebuf[MAX_MAILBOX_PATH];
     int fd;
@@ -434,9 +437,7 @@ struct mailbox *mailbox;
 /*
  * Copy the seen database from 'oldmailbox' to 'newmailbox'
  */
-int seen_copy(oldmailbox, newmailbox)
-struct mailbox *oldmailbox;
-struct mailbox *newmailbox;
+int seen_copy(struct mailbox *oldmailbox,struct mailbox *newmailbox)
 {
     char oldfname[MAX_MAILBOX_PATH];
     char newfname[MAX_MAILBOX_PATH];
@@ -538,12 +539,11 @@ int freeit;
  * calling 'report_proc' with 'report_rock' and a pointer to the line
  * in the database.
  */
-int seen_reconstruct(mailbox, report_time, prune_time, report_proc, report_rock)
-struct mailbox *mailbox;
-time_t report_time;
-time_t prune_time;
-int (*report_proc)();
-void *report_rock;
+int seen_reconstruct(struct mailbox *mailbox,
+		     time_t report_time,
+		     time_t prune_time,
+		     int (*report_proc)(),
+		     void *report_rock)
 {
     char fnamebuf[MAX_MAILBOX_PATH];
     char newfnamebuf[MAX_MAILBOX_PATH];
@@ -588,7 +588,7 @@ void *report_rock;
     newiov_num = 0;
 
     endline = base;
-    while (endline = memchr(line=endline, '\n', size - (endline - base))) {
+    while ((endline = memchr(line=endline, '\n', size - (endline - base)))) {
 	endline++;
 
 	/* Parse/check username */
@@ -602,7 +602,7 @@ void *report_rock;
 	/* Parse last-read timestamp */
 	p++;
 	lastread = 0;
-	while (p < endline && isdigit(*p)) {
+	while (p < endline && isdigit((int) *p)) {
 	    lastread = lastread * 10 + *p++ - '0';
 	}
 	if (p >= endline || *p++ != ' ') {
@@ -627,7 +627,7 @@ void *report_rock;
 	
 	/* Parse last-read uid */
 	lastuidread = 0;
-	while (p < endline && isdigit(*p)) {
+	while (p < endline && isdigit((int) *p)) {
 	    lastuidread = lastuidread * 10 + *p++ - '0';
 	}
 	if (p >= endline || *p++ != ' ' || lastuidread > uidtoobig) {
@@ -642,9 +642,9 @@ void *report_rock;
 	space = memchr(p, ' ', endline - p);
 
 	if (space && space+1 < endline &&
-	    space[0] == ' ' && isdigit(space[1])) {
+	    space[0] == ' ' && isdigit((int) space[1])) {
 	    /* Have a last-change timestamp */
-	    while (p < space && isdigit(*p)) {
+	    while (p < space && isdigit((int) *p)) {
 		lastchange = lastchange * 10 + *p++ - '0';
 	    }
 	    if (p != space) {
@@ -672,7 +672,7 @@ void *report_rock;
 
 	while (p < space) {
 	    thisuid = 0;
-	    while (p < space && isdigit(*p)) {
+	    while (p < space && isdigit((int) *p)) {
 		if (dst) *dst++ = *p;
 		thisuid = thisuid * 10 + *p++ - '0';
 	    }
@@ -680,7 +680,7 @@ void *report_rock;
 	    if (thisuid <= lastuid || thisuid > uidtoobig) {
 		/* Remove this UID and trailing separator */
 		FIXING();
-		while (isdigit(dst[-1])) dst--;
+		while (isdigit((int) dst[-1])) dst--;
 		if (dst[-1] == ':') dst[-1] = ',';
 	    }
 	    else if (lastsep == ':' && *p == ':') {
@@ -767,3 +767,10 @@ void *report_rock;
     return r;
 }
 
+/* done with all seen operations for this process */
+int seen_done(void)
+{
+    return 0;
+}
+
+
