@@ -1,6 +1,6 @@
 /* imtest.c -- imap test client
  * Tim Martin (SASL implementation)
- * $Id: imtest.c,v 1.58 2001/01/30 21:58:30 leg Exp $
+ * $Id: imtest.c,v 1.60 2001/02/19 19:32:06 leg Exp $
  *
  * Copyright (c) 1999-2000 Carnegie Mellon University.  All rights reserved.
  *
@@ -315,7 +315,7 @@ static int tls_init_clientengine(int verifydepth, char *var_tls_cert_file, char
     SSL_load_error_strings();
     SSLeay_add_ssl_algorithms();
 
-    tls_ctx = SSL_CTX_new(SSLv23_client_method());
+    tls_ctx = SSL_CTX_new(TLSv1_client_method());
     if (tls_ctx == NULL) {
 	return IMTEST_FAIL;
     };
@@ -782,10 +782,17 @@ int auth_sasl(char *mechlist)
   /* call sasl client start */
   while (saslresult==SASL_INTERACT)
   {
-    saslresult=sasl_client_start(conn, mechlist,
-				 NULL, &client_interact,
-				 &out, &outlen,
-				 &mechusing);
+#if (SASL_VERSION_MAJOR == 1) && (SASL_VERSION_MINOR == 5) && (SASL_VERSION_STEP < 26)
+    saslresult = sasl_client_start(conn, mechlist,
+				   NULL, &client_interact,
+				   &out, &outlen,
+				   &mechusing);
+#else
+    saslresult = sasl_client_start(conn, mechlist,
+				   NULL, &client_interact,
+				   NULL, NULL,
+				   &mechusing);
+#endif
     if (saslresult==SASL_INTERACT)
       fillin_interactions(client_interact); /* fill in prompts */      
 
